@@ -3,8 +3,8 @@
  *
  * Copyright (C) 2004 Jana Saout <jana@saout.de>
  * Copyright (C) 2004-2007 Clemens Fruhwirth <clemens@endorphin.org>
- * Copyright (C) 2009-2021 Red Hat, Inc. All rights reserved.
- * Copyright (C) 2009-2021 Milan Broz
+ * Copyright (C) 2009-2023 Red Hat, Inc. All rights reserved.
+ * Copyright (C) 2009-2023 Milan Broz
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -59,43 +59,33 @@ uint64_t crypt_getphysmemory_kb(void)
 	return phys_memory_kb;
 }
 
-/* MEMLOCK */
-#define DEFAULT_PROCESS_PRIORITY -18
-
-static int _priority;
-static int _memlock_count = 0;
-
-// return 1 if memory is locked
-int crypt_memlock_inc(struct crypt_device *ctx)
+void crypt_process_priority(struct crypt_device *cd, int *priority, bool raise)
 {
-	if (!_memlock_count++) {
-		log_dbg(ctx, "Locking memory.");
-		if (mlockall(MCL_CURRENT | MCL_FUTURE) == -1) {
-			log_dbg(ctx, "Cannot lock memory with mlockall.");
-			_memlock_count--;
-			return 0;
-		}
-		errno = 0;
-		if (((_priority = getpriority(PRIO_PROCESS, 0)) == -1) && errno)
-			log_err(ctx, _("Cannot get process priority."));
+	int _priority, new_priority;
+
+	if (raise) {
+		_priority = getpriority(PRIO_PROCESS, 0);
+		if (_priority < 0)
+			_priority = 0;
+		if (priority)
+			*priority = _priority;
+
+		/*
+		 * Do not bother checking CAP_SYS_NICE as device activation
+		 * requires CAP_SYSADMIN later anyway.
+		 */
+		if (getuid() || geteuid())
+			new_priority = 0;
 		else
-			if (setpriority(PRIO_PROCESS, 0, DEFAULT_PROCESS_PRIORITY))
-				log_dbg(ctx, "setpriority %d failed: %s",
-					DEFAULT_PROCESS_PRIORITY, strerror(errno));
-	}
-	return _memlock_count ? 1 : 0;
-}
+			new_priority = -18;
 
-int crypt_memlock_dec(struct crypt_device *ctx)
-{
-	if (_memlock_count && (!--_memlock_count)) {
-		log_dbg(ctx, "Unlocking memory.");
-		if (munlockall() == -1)
-			log_err(ctx, _("Cannot unlock memory."));
+		if (setpriority(PRIO_PROCESS, 0, new_priority))
+			log_dbg(cd, "Cannot raise process priority.");
+	} else {
+		_priority = priority ? *priority : 0;
 		if (setpriority(PRIO_PROCESS, 0, _priority))
-			log_dbg(ctx, "setpriority %d failed: %s", _priority, strerror(errno));
+			log_dbg(cd, "Cannot reset process priority.");
 	}
-	return _memlock_count ? 1 : 0;
 }
 
 /* Keyfile processing */
@@ -112,9 +102,9 @@ static int keyfile_seek(int fd, uint64_t bytes)
 	char tmp[BUFSIZ];
 	size_t next_read;
 	ssize_t bytes_r;
-	off64_t r;
+	off_t r;
 
-	r = lseek64(fd, bytes, SEEK_CUR);
+	r = lseek(fd, bytes, SEEK_CUR);
 	if (r > 0)
 		return 0;
 	if (r < 0 && errno != ESPIPE)
@@ -171,8 +161,7 @@ int crypt_keyfile_device_read(struct crypt_device *cd,  const char *keyfile,
 
 	if (isatty(fd)) {
 		log_err(cd, _("Cannot read keyfile from a terminal."));
-		r = -EINVAL;
-		goto out_err;
+		goto out;
 	}
 
 	/* If not requested otherwise, we limit input to prevent memory exhaustion */
@@ -180,7 +169,7 @@ int crypt_keyfile_device_read(struct crypt_device *cd,  const char *keyfile,
 		key_size = DEFAULT_KEYFILE_SIZE_MAXKB * 1024 + 1;
 		unlimited_read = 1;
 		/* use 4k for buffer (page divisor but avoid huge pages) */
-		buflen = 4096 - sizeof(size_t); // sizeof(struct safe_allocation);
+		buflen = 4096 - 16; /* sizeof(struct safe_allocation); */
 	} else
 		buflen = key_size;
 
@@ -188,7 +177,7 @@ int crypt_keyfile_device_read(struct crypt_device *cd,  const char *keyfile,
 	if (keyfile) {
 		if (stat(keyfile, &st) < 0) {
 			log_err(cd, _("Failed to stat key file."));
-			goto out_err;
+			goto out;
 		}
 		if (S_ISREG(st.st_mode)) {
 			regular_file = 1;
@@ -196,7 +185,7 @@ int crypt_keyfile_device_read(struct crypt_device *cd,  const char *keyfile,
 
 			if (keyfile_offset > file_read_size) {
 				log_err(cd, _("Cannot seek to requested keyfile offset."));
-				goto out_err;
+				goto out;
 			}
 			file_read_size -= keyfile_offset;
 
@@ -211,13 +200,13 @@ int crypt_keyfile_device_read(struct crypt_device *cd,  const char *keyfile,
 	pass = crypt_safe_alloc(buflen);
 	if (!pass) {
 		log_err(cd, _("Out of memory while reading passphrase."));
-		goto out_err;
+		goto out;
 	}
 
 	/* Discard keyfile_offset bytes on input */
 	if (keyfile_offset && keyfile_seek(fd, keyfile_offset) < 0) {
 		log_err(cd, _("Cannot seek to requested keyfile offset."));
-		goto out_err;
+		goto out;
 	}
 
 	for (i = 0, newline = 0; i < key_size; i += char_read) {
@@ -227,7 +216,7 @@ int crypt_keyfile_device_read(struct crypt_device *cd,  const char *keyfile,
 			if (!pass) {
 				log_err(cd, _("Out of memory while reading passphrase."));
 				r = -ENOMEM;
-				goto out_err;
+				goto out;
 			}
 		}
 
@@ -247,7 +236,7 @@ int crypt_keyfile_device_read(struct crypt_device *cd,  const char *keyfile,
 		if (char_read < 0) {
 			log_err(cd, _("Error reading passphrase."));
 			r = -EPIPE;
-			goto out_err;
+			goto out;
 		}
 
 		if (char_read == 0)
@@ -264,24 +253,24 @@ int crypt_keyfile_device_read(struct crypt_device *cd,  const char *keyfile,
 	if (!i && !regular_file && !newline) {
 		log_err(cd, _("Nothing to read on input."));
 		r = -EPIPE;
-		goto out_err;
+		goto out;
 	}
 
 	/* Fail if we exceeded internal default (no specified size) */
 	if (unlimited_read && i == key_size) {
 		log_err(cd, _("Maximum keyfile size exceeded."));
-		goto out_err;
+		goto out;
 	}
 
 	if (!unlimited_read && i != key_size) {
 		log_err(cd, _("Cannot read requested amount of data."));
-		goto out_err;
+		goto out;
 	}
 
 	*key = pass;
 	*key_size_read = i;
 	r = 0;
-out_err:
+out:
 	if (fd != STDIN_FILENO)
 		close(fd);
 
@@ -318,7 +307,7 @@ int kernel_version(uint64_t *kversion)
 	}
 
 	if (!r)
-		*kversion = version(maj, min, patch, rel);
+		*kversion = compact_version(maj, min, patch, rel);
 
 	return r;
 }
