@@ -3,8 +3,8 @@
  *
  * Copyright (C) 2004 Jana Saout <jana@saout.de>
  * Copyright (C) 2004-2007 Clemens Fruhwirth <clemens@endorphin.org>
- * Copyright (C) 2009-2021 Red Hat, Inc. All rights reserved.
- * Copyright (C) 2009-2021 Milan Broz
+ * Copyright (C) 2009-2023 Red Hat, Inc. All rights reserved.
+ * Copyright (C) 2009-2023 Milan Broz
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -22,7 +22,6 @@
  */
 
 #include "cryptsetup.h"
-#include <math.h>
 #include <signal.h>
 
 /* interrupt handling */
@@ -367,105 +366,6 @@ int tools_string_to_size(const char *s, uint64_t *size)
 	return 0;
 }
 
-/* Time progress helper */
-
-/* The difference in seconds between two times in "timeval" format. */
-static double time_diff(struct timeval *start, struct timeval *end)
-{
-	return (end->tv_sec - start->tv_sec)
-		+ (end->tv_usec - start->tv_usec) / 1E6;
-}
-
-static void tools_clear_line(void)
-{
-	/* vt100 code clear line */
-	log_std("\33[2K\r");
-}
-
-static void tools_time_progress(uint64_t device_size, uint64_t bytes, struct tools_progress_params *parms)
-{
-	struct timeval now_time;
-	unsigned long long mbytes, eta;
-	double tdiff, uib, frequency;
-	int final = (bytes == device_size);
-	const char *eol, *ustr = "";
-
-	gettimeofday(&now_time, NULL);
-	if (parms->start_time.tv_sec == 0 && parms->start_time.tv_usec == 0) {
-		parms->start_time = now_time;
-		parms->end_time = now_time;
-		parms->start_offset = bytes;
-		return;
-	}
-
-	if (parms->frequency) {
-		frequency = (double)parms->frequency;
-		eol = "\n";
-	} else {
-		frequency = 0.5;
-		eol = "";
-	}
-
-	if (!final && time_diff(&parms->end_time, &now_time) < frequency)
-		return;
-
-	parms->end_time = now_time;
-
-	tdiff = time_diff(&parms->start_time, &parms->end_time);
-	if (!tdiff)
-		return;
-
-	mbytes = bytes  / 1024 / 1024;
-	uib = (double)(bytes - parms->start_offset) / tdiff;
-
-	eta = (unsigned long long)(device_size / uib - tdiff);
-
-	if (uib > 1073741824.0f) {
-		uib /= 1073741824.0f;
-		ustr = "Gi";
-	} else if (uib > 1048576.0f) {
-		uib /= 1048576.0f;
-		ustr = "Mi";
-	} else if (uib > 1024.0f) {
-		uib /= 1024.0f;
-		ustr = "Ki";
-	}
-
-	if (!parms->frequency)
-		tools_clear_line();
-	if (final)
-		log_std("Finished, time %02llu:%02llu.%03llu, "
-			"%4llu MiB written, speed %5.1f %sB/s\n",
-			(unsigned long long)tdiff / 60,
-			(unsigned long long)tdiff % 60,
-			(unsigned long long)((tdiff - floor(tdiff)) * 1000.0),
-			mbytes, uib, ustr);
-	else
-		log_std("Progress: %5.1f%%, ETA %02llu:%02llu, "
-			"%4llu MiB written, speed %5.1f %sB/s%s",
-			(double)bytes / device_size * 100,
-			eta / 60, eta % 60, mbytes, uib, ustr, eol);
-	fflush(stdout);
-}
-
-int tools_wipe_progress(uint64_t size, uint64_t offset, void *usrptr)
-{
-	int r = 0;
-	struct tools_progress_params *parms = (struct tools_progress_params *)usrptr;
-
-	if (parms && !parms->batch_mode)
-		tools_time_progress(size, offset, parms);
-
-	check_signal(&r);
-	if (r) {
-		if (!parms || !parms->frequency)
-			tools_clear_line();
-		log_err(_("\nWipe interrupted."));
-	}
-
-	return r;
-}
-
 /*
  * Keyfile - is standard input treated as a binary file (no EOL handling).
  */
@@ -477,25 +377,7 @@ int tools_is_stdin(const char *key_file)
 	return strcmp(key_file, "-") ? 0 : 1;
 }
 
-int tools_reencrypt_progress(uint64_t size, uint64_t offset, void *usrptr)
-{
-	int r = 0;
-	struct tools_progress_params *parms = (struct tools_progress_params *)usrptr;
-
-	if (parms && !parms->batch_mode)
-		tools_time_progress(size, offset, parms);
-
-	check_signal(&r);
-	if (r) {
-		if (!parms || !parms->frequency)
-			tools_clear_line();
-		log_err(_("\nReencryption interrupted."));
-	}
-
-	return r;
-}
-
-int tools_read_mk(const char *file, char **key, int keysize)
+int tools_read_vk(const char *file, char **key, int keysize)
 {
 	int fd = -1, r = -EINVAL;
 
@@ -550,3 +432,37 @@ int tools_write_mk(const char *file, const char *key, int keysize)
 	close(fd);
 	return r;
 }
+
+void tools_package_version(const char *name, bool use_pwlibs)
+{
+	bool udev = false, blkid = false, keyring = false, fips = false;
+	bool kernel_capi = false, pwquality = false, passwdqc = false;
+#ifdef USE_UDEV
+	udev = true;
+#endif
+#ifdef HAVE_BLKID
+	blkid = true;
+#endif
+#ifdef KERNEL_KEYRING
+	keyring = true;
+#endif
+#ifdef ENABLE_FIPS
+	fips = true;
+#endif
+#ifdef ENABLE_AF_ALG
+	kernel_capi = true;
+#endif
+#if defined(ENABLE_PWQUALITY)
+	pwquality = true;
+#elif defined(ENABLE_PASSWDQC)
+	passwdqc = true;
+#endif
+	log_std("%s %s flags: %s%s%s%s%s%s%s\n", name, PACKAGE_VERSION,
+		udev ?	"UDEV " : "",
+		blkid ? "BLKID " : "",
+		keyring ? "KEYRING " : "",
+		fips ? "FIPS " : "",
+		kernel_capi ? "KERNEL_CAPI " : "",
+		pwquality && use_pwlibs ? "PWQUALITY " : "",
+		passwdqc && use_pwlibs ? "PASSWDQC " : "");
+}
